/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2018 MediaTek Inc.
 */

#ifndef _DT_BINDINGS_MT7629_POWER_H
#define _DT_BINDINGS_MT7629_POWER_H

#define MT7629_POWER_DOMAIN_ETHSYS	0
#define MT7629_POWER_DOMAIN_HIF0	1
#define MT7629_POWER_DOMAIN_HIF1	2

#endif /* _DT_BINDINGS_MT7629_POWER_H */
